/* $Id: cql3.y,v 1.2 2002/10/16 18:46:35 adam Exp $
 * yacc parser for CQL grammar that is sometimes referred to as "freeform"
 * with no restrictions on use paranthesis. Booleans are left assoc.
 *
 * This small program reads a CQL query on standard input and
 * produces a XCQL on standard output (unless an error occur)
 */
%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
    
typedef struct {
    struct XCQL_XML *xcql;
    char buf[80];
    size_t len;
    size_t max;
} token;        
#define YYSTYPE token

#include "xcqlutil.h"

int yylex(void);    
int yyerror(char *s);
%}

%token TERM AND OR NOT PROX EXACT ALL ANY GE LE NE

%%

top: cqlQuery { xcql_print($1.xcql); }
    ;

cqlQuery: 
  subQuery
|
  cqlQuery boolean subQuery {
      struct XCQL_XML *triple = xcql_create();
      struct XCQL_XML *bnode = xcql_create();

      xcql_add (bnode, "<boolean>", $2.buf, "</boolean>");

      xcql_add (triple, "<triple>", "", "");
      xcql_sub (triple, bnode);
      xcql_sub (triple, $1.xcql);
      xcql_sub (triple, $3.xcql);
      xcql_add (triple, "</triple>", "", "");
      $$.xcql = triple;
  }
;

subQuery:
  searchClause { 
      $$.xcql = $1.xcql;
  }
|
  '(' cqlQuery ')' {
      $$.xcql = $2.xcql;
  }
;

searchClause: 
  searchTerm {
      struct XCQL_XML *sc = xcql_create();
      struct XCQL_XML *sc3 = xcql_create();
      xcql_add (sc3, "<index>", "serverChoice", "</index>");
      xcql_add (sc3, "<relation>","=","</relation>");
      xcql_add (sc3, "<term>",$1.buf,"</term>");

      xcql_add (sc, "<searchClause>", "", "");
      xcql_sub (sc, sc3);
      xcql_add (sc, "</searchClause>", "", "");
      $$.xcql = sc;
  }
| 
  index relation searchTerm {
      struct XCQL_XML *sc = xcql_create();
      struct XCQL_XML *sc3 = xcql_create();

      xcql_add (sc3, "<index>", $1.buf, "</index>");
      xcql_add (sc3, "<relation>", $2.buf,"</relation>");
      xcql_add (sc3, "<term>",$3.buf,"</term>");

      xcql_add (sc, "<searchClause>", "", "");
      xcql_sub (sc, sc3);
      xcql_add (sc, "</searchClause>", "", "");
      $$.xcql = sc;
  }
;

boolean: 
  AND 
| OR 
| NOT
| PROX
;

relation: 
  '=' 
| '>' 
| '<'
| GE
| LE
| NE
| EXACT 
| ALL
| ANY
;

index: 
  searchTerm;

searchTerm:
  TERM
| AND
| OR
| NOT
| EXACT
| ALL
| ANY
| PROX
;

%%

int yyerror(char *s)
{
    puts (s);
    return 0;
}

#include "lexer.c"


int main()
{
#if YYDEBUG
    yydebug=1;
#endif
    
    yyparse();
    return 0;
}
