/*
 * $Id
 *
 * Utility for building XCQL. The C languages is not
 * well suited for this, so you'll probably be able to
 * achieve the same goal easier using a different languages.
 */
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "xcqlutil.h"

struct XCQL_XML *xcql_create (void)
{
    struct XCQL_XML *p = malloc (sizeof(*p));
    if (p)
    {
        p->lines = 0;
    }
    return p;
}
    
void xcql_sub(struct XCQL_XML *p, struct XCQL_XML *add)
{
    if (p)
    {
        struct XCQL_XML_line **lp = &p->lines;
        while (*lp)
            lp = &(*lp)->next;
        *lp = add->lines;
        while (*lp)
        {
            (*lp)->level ++;
            lp = &(*lp)->next;
        }
        free (add);
    }
}

static size_t xml_string (const char *src, char *dst)
{
    size_t i = 0;
    while (*src)
    {
        switch (*src)
        {
        case '&':
            if (dst)
                strcpy (dst + i, "&amp;");
	    i += 5;
	    break;
        case '<':
            if (dst)
                strcpy (dst + i, "&lt;");
	    i += 4;
	    break;
        case '>':
            if (dst)
                strcpy (dst + i, "&gt;");
	    i += 4;
	    break;
        case '"':
            if (dst)
                strcpy (dst + i, "&quot;");
	    i += 6;
	    break;
        case '\'':
            if (dst)
                strcpy (dst + i, "&apos;");
	    i += 6;
            break;
        default:
            if (dst)
                dst[i] = *src;
	    i++;
        }
	src++;
    } 
    if (dst)
        dst[i] = '\0';
    return i;
}

void xcql_add(struct XCQL_XML *p, const char *pre,  const char *mid,
              const char *post)
{
    if (p)
    {
        char *nstr;
        struct XCQL_XML_line **lp = &p->lines;
        while (*lp)
            lp = &(*lp)->next;
        *lp = malloc (sizeof(**lp));
        nstr = malloc (strlen(pre) + xml_string(mid, 0) + strlen(post) + 1);
        if (*lp)
        {
            if (nstr)
            {
                strcpy (nstr, pre);
		xml_string(mid, nstr+strlen(pre));
                strcat (nstr, post);
            }
            (*lp)->line = nstr;
            (*lp)->level = 0;
            (*lp)->next = 0;
        }
    }
}

void xcql_destroy (struct XCQL_XML *p)
{
    if (p)
    {
        struct XCQL_XML_line *l = p->lines;
        while (l)
        {
            struct XCQL_XML_line *tmp = l;
            l = l->next;
            free (tmp->line);
            free (tmp);
        }
        free (p);
    }
}

void xcql_print(struct XCQL_XML *p)
{
    if (p)
    {
        struct XCQL_XML_line *l = p->lines;
        for (; l; l = l->next)
            printf ("%*s %s\n", l->level * 2, "", l->line);
    }
}
