package Toolkit::Dump;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw(dump_hash dump_array dump_any);
$VERSION = '0.01';


sub default {
	my ($a, $b) = @_;
	return defined($a) ? $a : $b;
}


sub spaces {
	return "  " x shift;
}

sub dump_any {
  	my $e = shift;
	my $level = shift || 0;
	my $r = ref($e);
	###print "(e=$e r=$r)\n";
	if ( !$r ) {
	  print spaces($level),"'".$e."'","\n";
	} elsif ( $r eq 'SCALAR') {
	  print spaces($level),"(",$r,")\n";
	} elsif ( UNIVERSAL::isa($e, "ARRAY") ) {
	  my $row;
	  die "ARRAY NOT IMPLEMENTED YET\n";
	} elsif ( UNIVERSAL::isa($e, "HASH") ) {
	  print spaces($level++). $e ."{\n";
	  my %ha=%{$e};
	  my $key;
	  foreach $key (keys %ha ) {
	    print spaces($level),"$key => \n";
	    ##print spaces($level)," => \n";
	    dump_any($ha{$key},$level+1);
	  }
	  print spaces($level--),"}\n";
	} else {
	  print spaces($level), "??? $r $e \n";
	}
}

sub dump_hash {
	my $href = shift;
	my $key;
	my $level = shift || 0;
	
	print "{\n";
	foreach $key (keys %$href) {
		if (ref($href->{$key}) eq 'HASH') {
			printf("%s%6s	=>  ", spaces($level), $key);
			$level++;
			dump_hash($href->{$key}, $level);
			$level--;
		} elsif (ref($href->{$key}) eq 'ARRAY') {
			printf("%s%6s	=>  ", spaces($level), $key);
			$level++;
			dump_array($href->{$key}, $level);
			$level--;
		} else {
			printf("%s%6s	=>  %s\n", spaces($level), $key, default($href->{$key}, 'NULL'));
		}
	}
	print spaces($level), "}\n";
}

sub dump_array {
	my $aref = shift;
	my $level = shift || 0;
	my $entry;

	print "[\n";
	foreach $entry (@$aref) {
		if (ref($entry) eq 'HASH') {
			printf("%s", spaces($level));
			$level++;
			dump_hash($entry, $level);
			$level--;
		} elsif (ref($entry) eq 'ARRAY') {
			printf("%s", spaces($level));
			$level++;
			dump_array($entry, $level);
			$level--;
		} else {
			printf("%s%6s\n", spaces($level), default($entry, 'NULL'));
		}
	}
	print spaces($level), "]\n";
}
			

1;
__END__

=head1 NAME

Toolkit::Dump - Package with subroutines for dumping of Perl data structures

=head1 SYNOPSIS

  use Toolkit::Dump;

  my $hash_ref = {
			text	=>	'this is a text',
			list	=>	['first', 'second', 'third'],
			href	=>	{
						entry1	=>	['1st_elm', '2nd_elm'],
						entry2	=>	'just a string'
					},
	   	 };

  dump_hash($hash_ref);

=head1 DESCRIPTION

Here goes the description.

=head1 AUTHOR

Anders Snderberg Mortensen <sondberg@indexdata.dk>
Index Data ApS, Copenhagen, Denmark.
2001/03/02

=head1 SEE ALSO

perl(1).

=cut

#$Log: Dump.pm,v $
#Revision 1.2  2002/10/04 14:12:12  heikki
#hclient3 is my srw-client, that did not work
#
#Revision 1.1  2002/10/02 13:55:37  heikki
#Dumping results in srw-client
#
#Revision 1.3  2001/03/27 14:26:52  sondberg
#Included the 'DESCRIPTION' section into the POD.
#
#Revision 1.2  2001/03/07 15:33:13  sondberg
#Minor changes, started CVS logging into perl source.
#
